package com.ruoyi.project.pssr.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.pssr.mapper.TPssrMocMapper;
import com.ruoyi.project.pssr.domain.TPssrMoc;
import com.ruoyi.project.pssr.service.ITPssrMocService;

/**
 * MOC完成确认清单Service业务层处理
 *
 * @author ssy
 * @date 2024-10-31
 */
@Service
public class TPssrMocServiceImpl implements ITPssrMocService
{
    @Autowired
    private TPssrMocMapper tPssrMocMapper;

    /**
     * 查询MOC完成确认清单
     *
     * @param id MOC完成确认清单ID
     * @return MOC完成确认清单
     */
    @Override
    public TPssrMoc selectTPssrMocById(Long id)
    {
        return tPssrMocMapper.selectTPssrMocById(id);
    }

    @Override
    public int selectNoConfirmCount(Long subId)
    {
        return tPssrMocMapper.selectNoConfirmCount(subId);
    }
    /**
     * 查询MOC完成确认清单列表
     *
     * @param tPssrMoc MOC完成确认清单
     * @return MOC完成确认清单
     */
    @Override
    public List<TPssrMoc> selectTPssrMocList(TPssrMoc tPssrMoc)
    {
        return tPssrMocMapper.selectTPssrMocList(tPssrMoc);
    }

    /**
     * 新增MOC完成确认清单
     *
     * @param tPssrMoc MOC完成确认清单
     * @return 结果
     */
    @Override
    public int insertTPssrMoc(TPssrMoc tPssrMoc)
    {
        return tPssrMocMapper.insertTPssrMoc(tPssrMoc);
    }

    /**
     * 修改MOC完成确认清单
     *
     * @param tPssrMoc MOC完成确认清单
     * @return 结果
     */
    @Override
    public int updateTPssrMoc(TPssrMoc tPssrMoc)
    {
        return tPssrMocMapper.updateTPssrMoc(tPssrMoc);
    }

    /**
     * 批量删除MOC完成确认清单
     *
     * @param ids 需要删除的MOC完成确认清单ID
     * @return 结果
     */
    @Override
    public int deleteTPssrMocByIds(Long[] ids)
    {
        return tPssrMocMapper.deleteTPssrMocByIds(ids);
    }

    /**
     * 删除MOC完成确认清单信息
     *
     * @param id MOC完成确认清单ID
     * @return 结果
     */
    @Override
    public int deleteTPssrMocById(Long id)
    {
        return tPssrMocMapper.deleteTPssrMocById(id);
    }
}
